SystemOrganization addCategory: #'LDAP-Filters'!
SystemOrganization addCategory: #'LDAP-Exceptions'!
SystemOrganization addCategory: #'LDAP-BER'!
SystemOrganization addCategory: #'LDAP-Core'!
SystemOrganization addCategory: #'LDAP-Tests'!

TestCase subclass: #BERTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Tests'!

!BERTest class methodsFor: 'as yet unclassified' stamp: ''!
createLauncher
	| t1 |
	t1 := SimpleButtonMorph new.
	t1 target: BERTest.
	t1 actionSelector: #testAll.
	t1 position: World bounds bottomRight - (40 @ 40).
	t1 label: 'Test'.
	t1 openInWorld! !

!BERTest class methodsFor: 'as yet unclassified' stamp: ''!
getBooleanTestSet
	^ {{0. '010100'}. {1. '0101FF'}. {255. '0101FF'}. {1000. '0101FF'}. {false. '010100'}. {true. '0101FF'}}! !

!BERTest class methodsFor: 'as yet unclassified' stamp: ''!
getIntegerTestSet
	^ {{27066. '020269BA'}. {-27066. '02029646'}. {72. '020148'}. {127. '02017F'}. {-128. '020180'}. {128. '02020080'}. {0. '020100'}. {256. '02020100'}. {4294967290. '020500FFFFFFFA'}. {1. '020101'}. {-1. '0201FF'}. {-129. '0202FF7F'}}! !

!BERTest class methodsFor: 'as yet unclassified' stamp: ''!
isAbstract
	^ false! !

!BERTest class methodsFor: 'as yet unclassified' stamp: ''!
testAll
	| t1 |
	t1 := BERTest new.
	t1 testIntegerEncoding; testOctetStringEncoding; testSequenceEncoding; testBooleanEncoding; testIntegerDecoding; testAddRequest; testBindRequest; testDelRequest; testModifyRequest; testSearchRequest! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testAddRequest
	| t1 t2 |
	t2 := Dictionary new.
	t2 at: 'objectClass' put: (OrderedCollection new addLast: 'person';
			 yourself).
	t2 at: 'cn' put: (OrderedCollection new addLast: 'test2';
			 yourself).
	t2 at: 'sn' put: (OrderedCollection new addLast: 'test2';
			 yourself).
	t1 := LDAPEncoder
				addRequest: 1
				dn: 'cn=test2,dc=linalco,dc=test'
				attrs: t2.
	t1 := t1 asString asHex.
	Transcript show: 'testAddRequest got: ' , t1;
		 cr;
		 show: '-';
		 cr.
	self assert: t1 = '305B0201016856041B636E3D74657374322C64633D6C696E616C636F2C64633D7465737430373017040B6F626A656374436C61737331080406706572736F6E300D0402636E310704057465737432300D0402736E310704057465737432'! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testBindRequest
	| t1 |
	t1 := LDAPEncoder
				bindRequest: 1
				username: 'cn=admin,dc=linalco,dc=test'
				credentials: 'secret'
				method: nil.
	t1 := t1 asString asHex.
	Transcript show: 'testBindRequest got: ' , t1;
		 cr.
	self assert: t1 = '302D0201016028020103041B636E3D61646D696E2C64633D6C696E616C636F2C64633D746573748006736563726574'! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testBooleanEncoding
	| t1 t2 t3 t4 |
	t1 := BERBoolean new.
	t2 := ReadWriteStream on: String new.
	t3 := self class getBooleanTestSet.
	t3
		do: [:t5 | 
			t2 resetToStart.
			t1
				value: (t5 at: 1).
			t1 writeOn: t2.
			t4 := t2 contents asString asHex.
			Transcript show: 'Expected: '
					, (t5 at: 2) , ' Got: ' , t4;
				 cr.
			self assert: t4
					= (t5 at: 2)]! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testDelRequest
	| t1 |
	t1 := LDAPEncoder delRequest: 1 dn: 'cn=test2,dc=linalco,dc=test'.
	t1 := t1 asString asHex.
	Transcript show: 'testDelRequest got: ' , t1;
		 cr;
		 show: '-';
		 cr.
	self assert: t1 = '30200201014A1B636E3D74657374322C64633D6C696E616C636F2C64633D74657374'! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testIntegerDecoding
	| t1 t2 t3 t4 t5 t6 |
	t2 := ReadWriteStream on: String new.
	t3 := self class getIntegerTestSet.
	t3
		do: [:t7 | 
			t5 := ReadStream
						on: (t7 at: 2).
			t4 := t7 at: 1.
			t2 resetToStart.
			[t5 atEnd]
				whileFalse: [t6 := t5 next digitValue * 16.
					t6 := t6 + t5 next digitValue.
					t2 nextPut: t6 asCharacter].
			t2 reset.
			t1 := BERInteger newFrom: t2.
			self assert: t1 class = BERInteger.
			Transcript show: 'Expected: ' , t4 asString , ' Got: ' , t1 value asString;
				 cr.
			self assert: t1 value = t4]! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testIntegerEncoding
	| t1 t2 t3 t4 |
	t1 := BERInteger new.
	t2 := ReadWriteStream on: String new.
	t3 := self class getIntegerTestSet.
	t3
		do: [:t5 | 
			t2 resetToStart.
			t1
				value: (t5 at: 1).
			t1 writeOn: t2.
			t4 := t2 contents asString asHex.
			Transcript show: 'Expected: '
					, (t5 at: 2) , ' Got: ' , t4;
				 cr.
			self assert: t4
					= (t5 at: 2)]! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testModifyRequest
	| t1 t2 |
	t2 := {LDAPAttrModifier set: 'sn' to: {'test5sn'. 'foo'. 'bar'}. LDAPAttrModifier addTo: 'description' values: {'rchueo'}}.
	t1 := LDAPEncoder
				modifyRequest: 1
				dn: 'cn=test5,dc=linalco,dc=test'
				ops: t2.
	t1 := t1 asString asHex.
	Transcript show: 'testModifyRequest got: ' , t1;
		 cr! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testOctetStringEncoding
	| t1 t2 t3 |
	t1 := BEROctetString new.
	t2 := ReadWriteStream on: String new.
	t3 := {{'hello'. 5. '040568656C6C6F'}}.
	t3
		do: [:t4 | 
			t2 resetToStart.
			t1
				value: (t4 at: 1).
			t1 writeOn: t2.
			self assert: t2 contents asString asHex
					= (t4 at: 3)]! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testSearchRequest
	| t1 |
	t1 := LDAPEncoder
				searchRequest: 1
				base: 'dc=linalco, dc=test'
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter with: 'objectclass')
				attrs: OrderedCollection new
				wantAttrsOnly: false.
	t1 := t1 asString asHex.
	Transcript show: 'testSearchRequest got: ' , t1;
		 cr.
	self assert: t1 = '30380201016333041364633D6C696E616C636F2C2064633D746573740A01020A0100020100020100010100870B6F626A656374636C6173733000'! !

!BERTest methodsFor: 'as yet unclassified' stamp: ''!
testSequenceEncoding
	| t1 t2 t3 t4 |
	t1 := BERSequence new.
	t2 := BERInteger new value: 17.
	t3 := BERInteger new value: 170.
	t1 addElement: t2.
	t1 addElement: t3.
	t4 := ReadWriteStream on: String new.
	t1 writeOn: t4.
	self assert: t4 contents asString asHex = '3007020111020200AA'! !

TestCase subclass: #LDAPTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Tests'!

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
baseDN
	^ 'dc=linalco,dc=test'! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
bindDN
	^ 'cn=admin, ' , self baseDN! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
hostname
	^ '127.0.0.1'! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
newSuite
	| t1 |
	t1 := TestSuite named: 'LDAP'.
	t1
		addTest: (LDAPTest selector: #testSearch).
	^ t1! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
password
	^ 'secret'! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
port
	^ 12000! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: ''!
testAll
	| t1 |
	t1 := LDAPTest new.
	t1 testDoubleBind; testAddWithCollectionValues; testSearch; testSearchWithNext; testSearchWithFilters; testDel; testAddWithStringValues; testSearch; testCompare; testModify; testSearch; testDel! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
checkSearchResults: t1 expecting: t2 
	| t4 t5 |
	t1 size = t2 size
		ifFalse: [^ false].
	t2
		do: [:t3 | 
			t3
				ifNil: [t4 := self class baseDN]
				ifNotNil: [t4 := t3 , ',' , self class baseDN].
			t5 := false.
			t1
				do: [:t6 | t6 dn = t4
						ifTrue: [t5 := true]].
			t5
				ifFalse: [^ false]].
	^ true! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
connect
	| t1 t2 |
	t2 := LDAPConnection to: self class hostname port: self class port.
	t1 := t2 bindAs: self class bindDN credentials: self class password.
	t1 wait.
	^ t2! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testAddWithCollectionValues
	| t1 t2 t3 t5 |
	t1 := self connect.
	(1 to: 5)
		do: [:t4 | 
			t5 := 'test' , t4 asString.
			t3 := Dictionary new at: 'objectClass' put: (OrderedCollection new add: 'person';
							 yourself);
						 at: 'cn' put: (OrderedCollection new add: t5;
							 yourself);
						 at: 'sn' put: (OrderedCollection new add: t5;
							 yourself);
						 yourself.
			t2 := t1 addEntry: 'cn=' , t5 , ',' , self class baseDN attrs: t3.
			t2 wait]! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testAddWithStringValues
	| t1 t2 t3 t5 |
	t1 := self connect.
	(1 to: 5)
		do: [:t4 | 
			t5 := 'test' , t4 asString.
			t3 := Dictionary new at: 'objectClass' put: 'person';
						 at: 'cn' put: t5;
						 at: 'sn' put: t5;
						 yourself.
			t2 := t1 addEntry: 'cn=' , t5 , ',' , self class baseDN attrs: t3.
			t2 wait]! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testBind
	| t1 |
	t1 := self connect.
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testCompare
	| t1 t2 |
	t1 := self connect.
	t2 := t1
				compare: 'cn=test1,' , self class baseDN
				using: (LDAPFilterEquality new setType: 'cn' value: 'test1').
	t2 wait.
	self assert: t2 result.
	t2 := t1
				compare: 'cn=test1,' , self class baseDN
				using: (LDAPFilterEquality new setType: 'cn' value: 'test2').
	t2 wait.
	self deny: t2 result.
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testDel
	| t1 t2 t4 |
	t1 := self connect.
	(1 to: 5)
		do: [:t3 | 
			t4 := 'test' , t3 asString.
			t2 := t1 delEntry: 'cn=' , t4 , ',' , self class baseDN.
			t2 wait].
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testDoubleBind
	| t1 t2 |
	t1 := self connect.
	t2 := t1 bindAs: self class bindDN credentials: self class password.
	t2 wait.
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testModify
	| t1 t2 t3 t4 |
	t1 := self connect.
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter with: 'cn' equalTo: 'test1')
				attrs: nil
				wantAttrsOnly: false.
	t2 wait.
	t3 := t2 result at: 1.
	t4 := {LDAPAttrModifier set: 'sn' to: {'test5sn'. 'foo'. 'bar'}. LDAPAttrModifier addTo: 'description' values: {'rchueo'}}.
	t2 := t1 modify: t3 dn with: t4.
	t2 wait.
	t2 := t1
				compare: 'cn=test1,' , self class baseDN
				using: (LDAPFilterEquality new setType: 'sn' value: 'test5sn').
	t2 wait.
	self assert: t2 result.
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testSearch
	| t1 t2 t3 |
	t1 := self connect.
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: nil
				attrs: nil
				wantAttrsOnly: false.
	t3 := t2 result.
	self
		assert: (self checkSearchResults: t3 expecting: {nil. 'cn=test1'. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5'}).
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testSearchWithFilters
	| t1 t2 t3 |
	t1 := self connect.
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter with: 'objectclass')
				attrs: nil
				wantAttrsOnly: false.
	t3 := t2 result.
	self
		assert: (self checkSearchResults: t3 expecting: {nil. 'cn=test1'. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5'}).
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter with: 'cn' equalTo: 'test1')
				attrs: nil
				wantAttrsOnly: false.
	t3 := t2 result.
	self
		assert: (self checkSearchResults: t3 expecting: {'cn=test1'}).
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter
						not: (LDAPFilter with: 'cn' equalTo: 'test1'))
				attrs: nil
				wantAttrsOnly: false.
	t3 := t2 result.
	self
		assert: (self checkSearchResults: t3 expecting: {nil. 'cn=test2'. 'cn=test3'. 'cn=test4'. 'cn=test5'}).
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: (LDAPFilter andOf: (OrderedCollection new
							add: (LDAPFilter
									not: (LDAPFilter with: 'cn' equalTo: 'test1'));
							
							add: (LDAPFilter
									not: (LDAPFilter with: 'sn' equalTo: 'test2'));
							 yourself))
				attrs: nil
				wantAttrsOnly: false.
	t3 := t2 result.
	self
		assert: (self checkSearchResults: t3 expecting: {nil. 'cn=test3'. 'cn=test4'. 'cn=test5'}).
	t1 disconnect! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: ''!
testSearchWithNext
	| t1 t2 t3 t4 |
	t1 := self connect.
	t2 := t1
				newSearch: self class baseDN
				scope: LDAPConnection wholeSubtree
				deref: LDAPConnection derefNever
				filter: nil
				attrs: nil
				wantAttrsOnly: false.
	t3 := OrderedCollection new.
	[(t4 := t2 next) == nil]
		whileFalse: [t3 add: t4].
	t1 disconnect! !

Object subclass: #BERElement
	instanceVariableNames: 'length lengthLength value tagHeader tagLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

BERElement subclass: #BERBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BERBoolean class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 1! !

!BERBoolean methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	value := t1 next.
	value := value > 0! !

!BERBoolean methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	self writeLength: 1 on: t1.
	(value = 0
			or: [value = false])
		ifTrue: [t1 nextPut: 0 asCharacter]
		ifFalse: [t1 nextPut: 255 asCharacter]! !

BERElement subclass: #BERConstruct
	instanceVariableNames: 'elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BERConstruct class methodsFor: 'as yet unclassified' stamp: ''!
new
	^ self basicNew initialize! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
addElement: t1 
	self addElement: t1 withTag: nil! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
addElement: t1 withTag: t2 
	| t3 |
	t3 := Association new; key: t2 value: t1.
	elements addLast: t3! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	| t2 t3 |
	t2 := self length.
	[t2 > 0]
		whileTrue: [t3 := self class newFrom: t1.
			t2 := t2 - t3 totalLength.
			self addElement: t3]! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
elements
	^ unknown5! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
initialize
	super initialize.
	elements := OrderedCollection new! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	| t2 t3 |
	t2 := Array new.
	t3 := WriteStream on: t2.
	elements
		do: [:t4 | t4 value writeOn: t3 withTag: t4 key].
	self writeLength: t3 contents size on: t1.
	t1 nextPutAll: t3 contents! !

BERConstruct subclass: #BERSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BERSequence class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 16 + 32! !

!BERSequence methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	^ super decode: t1! !

BERConstruct subclass: #BERSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BERSet class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 17 + 32! !

!BERSet methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	^ super decode: t1! !

!BERElement class methodsFor: 'as yet unclassified' stamp: ''!
elementClasses
	^ {BERInteger. BEROctetString. BERSequence. BEREnumerated. BERBoolean. BERSet. BERNull. LDAPAddResult. LDAPBindResult. LDAPCompareResult. LDAPDelResult. LDAPExtendedResult. LDAPModifyResult. LDAPModifyDNResult. LDAPSearchResultEntry. LDAPSearchResultCode}! !

!BERElement class methodsFor: 'as yet unclassified' stamp: ''!
identifyIncomingElement: t1 
	| t2 |
	t2 := self elementClasses
				detect: [:t3 | t3 tagValue = t1 asInteger]
				ifNone: [].
	^ t2! !

!BERElement class methodsFor: 'as yet unclassified' stamp: ''!
new
	^ self basicNew initialize! !

!BERElement class methodsFor: 'as yet unclassified' stamp: ''!
newFrom: t1 
	| t2 t3 t4 |
	t2 := t1 next.
	t4 := self identifyIncomingElement: t2.
	t4
		ifNil: [LDAPException new messageText: 'invalid tag -- make sure class is in identifyIncomingElements';
				 signal].
	t3 := t4 new setTag: t2.
	t3 readLengthFrom: t1.
	t3 decode: t1.
	^ t3! !

!BERElement class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	self subclassResponsibility! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	self subclassResponsibility! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
initialize
	tagHeader := 0! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
length
	^ unknown0! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
lengthLength
	^ unknown1! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
readLengthFrom: t1 
	| t2 t3 |
	t3 := t1 next asInteger.
	t3 < 128
		ifTrue: [length := t3.
			lengthLength := 1]
		ifFalse: [t2 := t1
						next: (t3 bitXor: 128).
			lengthLength := (t3 bitXor: 128)
						+ 1.
			length := t2 contents
						inject: 0
						into: [:t4 :t5 | (t4 bitShift: 8)
								+ t5 asInteger]].
	^ length! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
setTag: t1 
	tagHeader := t1.
	tagLength := 1! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
tagLength
	^ 1! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
tagSetApplication
	tagHeader := tagHeader bitOr: 64! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
tagSetContext
	tagHeader := tagHeader bitOr: 128! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
totalLength
	^ self length + self lengthLength + self tagLength! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
value
	^ unknown2! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
value: t1 
	value := t1! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	self subclassResponsibility! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
writeLength: t1 on: t2 
	| t3 t4 t5 t6 |
	t4 := 1.
	t1 < 128
		ifTrue: [t2 nextPut: t1 asCharacter]
		ifFalse: [t3 := ByteArray new: t1 digitLength + 1.
			t5 := t1.
			[t5 > 0]
				whileTrue: [t3
						at: t4
						put: (t5 bitAnd: 255).
					t4 := t4 + 1.
					t5 := t5 bitShift: -8].
			t3
				at: t4
				put: (t4 - 1 bitOr: 128).
			t6 := ByteArray new: t4.
			(1 to: t4)
				do: [:t7 | t6
						at: t7
						put: (t3 at: t4 + 1 - t7)].
			t2 nextPutAll: t6 asString].
	^ t4! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
writeOn: t1 
	t1 nextPut: (self class tagValue bitOr: tagHeader) asCharacter.
	self writeBodyOn: t1! !

!BERElement methodsFor: 'as yet unclassified' stamp: ''!
writeOn: t1 withTag: t2 
	| t3 |
	t3 := self class tagValue bitOr: tagHeader.
	t2
		ifNotNil: [t3 := (t3 bitOr: 31)
						bitXor: 31.
			t3 := t3 bitOr: t2].
	t1 nextPut: t3 asCharacter.
	self writeBodyOn: t1! !

BERElement subclass: #BERInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

BERInteger subclass: #BEREnumerated
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BEREnumerated class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 10! !

!BEREnumerated methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	super decode: t1! !

!BERInteger class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 2! !

!BERInteger methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	| t2 t3 |
	value := 0.
	(1 to: length)
		do: [:t4 | value := (value bitShift: 8)
						+ t1 next asInteger].
	t2 := value highBit.
	t2 = (length * 8)
		ifTrue: [t3 := (1 bitShift: t2)
						- 1.
			value := value bitXor: t3.
			value := (value + 1) negated]! !

!BERInteger methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	| t2 t3 t4 t5 t6 |
	t4 := value < 0.
	t2 := ByteArray new: value digitLength + 2.
	t4 not
		ifTrue: [value = 0
				ifTrue: [t6 := 1.
					t2 at: t6 put: 0]
				ifFalse: [t5 := value.
					t6 := 0.
					[t5 > 0]
						whileTrue: [t6 := t6 + 1.
							t2
								at: t6
								put: (t5 bitAnd: 255).
							t5 := t5 bitShift: -8].
					nil]]
		ifFalse: [t5 := value negated.
			t5 := t5 - 1.
			t6 := 0.
			[t6 := t6 + 1.
			t2
				at: t6
				put: ((t5 bitXor: 255)
						bitAnd: 255).
			t5 := t5 bitShift: -8.
			t5 > 0] whileTrue.
			nil].
	value > 0
		ifTrue: [((t2 at: t6)
					bitAnd: 128)
					> 0
				ifTrue: [t6 := t6 + 1.
					t2 at: t6 put: 0]].
	value < 0
		ifTrue: [((t2 at: t6)
					bitAnd: 128)
					= 0
				ifTrue: [t6 := t6 + 1.
					t2 at: t6 put: 255]].
	t3 := ByteArray new: t6.
	(1 to: t6)
		do: [:t7 | t3
				at: t7
				put: (t2 at: t6 + 1 - t7)].
	self writeLength: t6 on: t1.
	t1 nextPutAll: t3 asString.
	^ t6! !

BERElement subclass: #BERNull
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BERNull class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 5! !

!BERNull methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	value := nil! !

!BERNull methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	self writeLength: 0 on: t1! !

BERElement subclass: #BEROctetString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BER'!

!BEROctetString class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 4! !

!BEROctetString methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	value := t1 next: length! !

!BEROctetString methodsFor: 'as yet unclassified' stamp: ''!
writeBodyOn: t1 
	self writeLength: value size on: t1.
	t1 nextPutAll: value.
	^ value size! !

BERElement subclass: #LDAPElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPElement class methodsFor: 'as yet unclassified' stamp: ''!
elementClasses
	^ super elementClasses , {LDAPAddResult. LDAPBindResult. LDAPCompareResult. LDAPDelResult. LDAPExtendedResult. LDAPModifyResult. LDAPModifyDNResult. LDAPSearchResultEntry. LDAPSearchResultCode}! !

BERElement subclass: #LDAPMessage
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPMessage class methodsFor: 'as yet unclassified' stamp: ''!
newFrom: t1 
	| t2 t3 |
	t2 := BERElement newFrom: t1.
	t3 := (t2 elements at: 2) value.
	t3 id: (t2 elements at: 1) value value.
	^ t3! !

!LDAPMessage methodsFor: 'as yet unclassified' stamp: ''!
id
	^ unknown5! !

!LDAPMessage methodsFor: 'as yet unclassified' stamp: ''!
id: t1 
	id := t1! !

LDAPMessage subclass: #LDAPResult
	instanceVariableNames: 'resultCode errorMessage matchedDN referral'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

LDAPResult subclass: #LDAPAddResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPAddResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 105! !

LDAPResult subclass: #LDAPBindResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPBindResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 97! !

LDAPResult subclass: #LDAPCompareResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPCompareResult class methodsFor: 'as yet unclassified' stamp: ''!
missingRFC2251
	self notYetImplemented! !

!LDAPCompareResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 111! !

LDAPResult subclass: #LDAPDelResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPDelResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 107! !

LDAPResult subclass: #LDAPExtendedResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPExtendedResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 120! !

LDAPResult subclass: #LDAPModifyDNResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyDNResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 109! !

LDAPResult subclass: #LDAPModifyResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 103! !

!LDAPResult class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	self subclassResponsibility! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: ''!
checkForExceptions
	| t1 |
	(resultCode = 0
			or: [resultCode = 5]
			or: [resultCode = 6])
		ifFalse: [t1 := LDAPException newWithCode: resultCode.
			t1
				signal: (errorMessage
						ifEmpty: [t1 messageText])]! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	| t2 t3 t4 t5 t6 |
	t2 := BERElement newFrom: t1.
	t4 := BERElement newFrom: t1.
	t3 := BERElement newFrom: t1.
	resultCode := t2 value.
	errorMessage := t3 value.
	matchedDN := t4 value.
	t5 := t2 totalLength + t4 totalLength + t3 totalLength.
	t5 - self length ~= 0
		ifTrue: [t6 := BERElement newFrom: t1.
			referral := t6 value.
			self notYetImplemented]! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: ''!
referral
	^ unknown9! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: ''!
referral: t1 
	referral := t1! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: ''!
resultCode
	^ unknown6! !

LDAPResult subclass: #LDAPSearchResultCode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPSearchResultCode class methodsFor: 'as yet unclassified' stamp: ''!
isEndMarker
	^ true! !

!LDAPSearchResultCode class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 101! !

LDAPMessage subclass: #LDAPSearchResultEntry
	instanceVariableNames: 'dn attrs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPSearchResultEntry class methodsFor: 'as yet unclassified' stamp: ''!
isEndMarker
	^ false! !

!LDAPSearchResultEntry class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 100! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: ''!
attrAt: t1 
	| t2 |
	t2 := attrs
				at: t1 asByteArray
				ifAbsent: [^ nil].
	^ t2 first asString! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: ''!
attrs
	^ unknown7! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: ''!
decode: t1 
	| t2 t3 t5 |
	t2 := BERElement newFrom: t1.
	t3 := BERElement newFrom: t1.
	attrs := Dictionary new.
	t3 elements
		do: [:t4 | 
			t5 := OrderedCollection new.
			(t4 value elements at: 2) value elements
				do: [:t6 | t5 add: t6 value value asString].
			attrs at: (t4 value elements at: 1) value value asString put: t5].
	dn := t2 value asString! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: ''!
dn
	^ unknown6! !

Object subclass: #LDAPAttrModifier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPAttrModifier class methodsFor: 'as yet unclassified' stamp: ''!
addTo: t1 values: t2 
	^ LDAPModifyAddOp affecting: t1 values: t2! !

!LDAPAttrModifier class methodsFor: 'as yet unclassified' stamp: ''!
del: t1 
	^ LDAPModifyDelOp affecting: t1 values: {}! !

!LDAPAttrModifier class methodsFor: 'as yet unclassified' stamp: ''!
delFrom: t1 values: t2 
	^ LDAPModifyDelOp affecting: t1 values: t2! !

!LDAPAttrModifier class methodsFor: 'as yet unclassified' stamp: ''!
set: t1 to: t2 
	^ LDAPModifyReplaceOp affecting: t1 values: t2! !

Object subclass: #LDAPConnection
	instanceVariableNames: 'worker workerThread'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPConnection class methodsFor: 'search scope constants' stamp: ''!
baseObject
	^ 0! !

!LDAPConnection class methodsFor: 'as yet unclassified' stamp: ''!
collectionizeAttrs: t1 
	| t2 t4 |
	t2 := t1 clone.
	t2 keys
		do: [:t3 | 
			t4 := t1 at: t3.
			(t4 isKindOf: String)
				ifTrue: [t1 at: t3 put: (OrderedCollection new add: t4;
							 yourself)]].
	^ t2! !

!LDAPConnection class methodsFor: 'search deref constants' stamp: ''!
derefAlways
	^ 3! !

!LDAPConnection class methodsFor: 'search deref constants' stamp: ''!
derefFindingBaseObj
	^ 2! !

!LDAPConnection class methodsFor: 'search deref constants' stamp: ''!
derefInSearching
	^ 1! !

!LDAPConnection class methodsFor: 'search deref constants' stamp: ''!
derefNever
	^ 0! !

!LDAPConnection class methodsFor: 'documentation' stamp: ''!
missingRFC2251
	self notYetImplemented! !

!LDAPConnection class methodsFor: 'search scope constants' stamp: ''!
singleLevel
	^ 1! !

!LDAPConnection class methodsFor: 'as yet unclassified' stamp: ''!
to: t1 
	^ self to: t1 port: 389! !

!LDAPConnection class methodsFor: 'as yet unclassified' stamp: ''!
to: t1 port: t2 
	| t3 |
	t3 := self basicNew initializeWithHost: t1 port: t2.
	^ t3! !

!LDAPConnection class methodsFor: 'search scope constants' stamp: ''!
wholeSubtree
	^ 2! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
addEntry: t1 attrs: t2 
	^ LDAPAddRequest new
		initializeWith: worker
		dn: t1
		attrs: (self class collectionizeAttrs: t2)! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
bindAs: t1 credentials: t2 
	^ self
		bindAs: t1
		credentials: t2
		method: nil! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
bindAs: t1 credentials: t2 method: t3 
	^ LDAPBindRequest new
		initializeWith: worker
		username: t1
		credentials: t2
		method: t3! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
compare: t1 using: t2 
	^ LDAPCompareRequest new
		initializeWith: worker
		dn: t1
		comparing: t2! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
delEntry: t1 
	^ LDAPDelRequest new initializeWith: worker dn: t1! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
disconnect
	self unbind.
	worker disconnect.
	workerThread terminate! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
initializeWithHost: t1 port: t2 
	worker := LDAPWorker newTo: t1 port: t2.
	worker connect.
	workerThread := worker start! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
isValid
	^ worker running! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
modify: t1 with: t2 
	^ LDAPModifyRequest new
		initializeWith: worker
		dn: t1
		ops: t2! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
modifyDN: t1 toRDN: t2 moveUnderDN: t3 wantToClearAttrs: t4 
	^ LDAPModifyDNRequest new
		initializeWith: worker
		modifyDN: t1
		toRDN: t2
		underDN: t3
		wantToClearAttrs: t4! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
modifyDN: t1 toRDN: t2 wantToClearAttrs: t3 
	^ self
		modifyDN: t1
		toRDN: t2
		moveUnderDN: nil
		wantToClearAttrs: t3! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
newSearch: t1 scope: t2 deref: t3 filter: t4 attrs: t5 wantAttrsOnly: t6 
	| t7 t8 |
	t7 := t4
				ifNil: [t7 := LDAPFilter with: 'objectclass']
				ifNotNil: [t4].
	t8 := t5
				ifNil: [t8 := OrderedCollection new]
				ifNotNil: [t5].
	^ LDAPSearchRequest new
		initializeWith: worker
		base: t1
		scope: t2
		deref: t3
		filter: t7
		attrs: t8
		wantAttrsOnly: t6! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: ''!
unbind
	LDAPUnbindRequest new initializeWith: worker! !

Object subclass: #LDAPEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
abandonRequest: t1 abandonId: t2 
	| t3 t4 t5 |
	t3 := ReadWriteStream on: String new.
	t4 := BERSequence new.
	t4
		addElement: (BERInteger new value: t1).
	t5 := BERInteger new value: t2.
	t4 addElement: t5 tagSetApplication withTag: 16.
	t4 writeOn: t3.
	^ t3 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
addRequest: t1 dn: t2 attrs: t3 
	| t4 t5 t6 |
	t4 := ReadWriteStream on: String new.
	t5 := BERSequence new.
	t5
		addElement: (BERInteger new value: t1).
	t6 := BERSequence new.
	t6
		addElement: (BEROctetString new value: t2).
	t6
		addElement: (self attrsAsBERElement: t3).
	t5 addElement: t6 tagSetApplication withTag: 8.
	t5 writeOn: t4.
	^ t4 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
attrAsBERElement: t1 
	| t2 t3 t4 |
	t2 := BERSequence new.
	t4 := t1 key.
	t2
		addElement: (BEROctetString new value: t4).
	t3 := BERSet new.
	t1 value
		do: [:t5 | t3
				addElement: (BEROctetString new value: t5)].
	t2 addElement: t3.
	^ t2! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
attrsAsBERElement: t1 
	| t2 |
	t2 := BERSequence new.
	t1
		associationsDo: [:t3 | t2
				addElement: (self attrAsBERElement: t3)].
	^ t2! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
bindRequest: t1 username: t2 credentials: t3 method: t4 
	| t5 t6 t7 |
	t5 := ReadWriteStream on: String new.
	t6 := BERSequence new.
	t6
		addElement: (BERInteger new value: t1).
	t7 := BERSequence new tagSetApplication.
	t7
		addElement: (BERInteger new value: 3).
	t7
		addElement: (BEROctetString new value: t2).
	t7 addElement: (BEROctetString new tagSetContext; value: t3) withTag: 0.
	t6 addElement: t7 withTag: 0.
	t6 writeOn: t5.
	^ t5 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
compareRequest: t1 dn: t2 comparing: t3 
	| t4 t5 t6 |
	t4 := ReadWriteStream on: String new.
	t5 := BERSequence new.
	t5
		addElement: (BERInteger new value: t1).
	t6 := BERSequence new.
	t6
		addElement: (BEROctetString new value: t2);
		 addElement: t3 asBERElement.
	t5 addElement: t6 tagSetApplication withTag: 14.
	t5 writeOn: t4.
	^ t4 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
delRequest: t1 dn: t2 
	| t3 t4 t5 |
	t3 := ReadWriteStream on: String new.
	t4 := BERSequence new.
	t4
		addElement: (BERInteger new value: t1).
	t5 := BEROctetString new value: t2.
	t4 addElement: t5 tagSetApplication withTag: 10.
	t4 writeOn: t3.
	^ t3 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
modifyDNRequest: t1 dn: t2 toRDN: t3 underDN: t4 wantToClearAttrs: t5 
	| t6 t7 t8 |
	t6 := ReadWriteStream on: String new.
	t7 := BERSequence new.
	t7
		addElement: (BERInteger new value: t1).
	t8 := BERSequence new.
	t8
		addElement: (BEROctetString new value: t2);
		
		addElement: (BEROctetString new value: t3);
		
		addElement: (BERBoolean new value: t5).
	t4
		ifNotNil: [t8
				addElement: (BEROctetString new value: t4)
				withTag: 0].
	t7 addElement: t8 tagSetApplication withTag: 12.
	t7 writeOn: t6.
	^ t6 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
modifyRequest: t1 dn: t2 ops: t3 
	| t4 t5 t6 t7 |
	t4 := ReadWriteStream on: String new.
	t5 := BERSequence new.
	t5
		addElement: (BERInteger new value: t1).
	t6 := BERSequence new.
	t6
		addElement: (BEROctetString new value: t2).
	t7 := BERSequence new.
	t3
		do: [:t8 | t7 addElement: t8 asBERElement].
	t6 addElement: t7.
	t5 addElement: t6 tagSetApplication withTag: 6.
	t5 writeOn: t4.
	^ t4 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
newFrom: t1 
	| t2 t3 t4 |
	t2 := t1 next.
	t4 := self identifyIncomingElement: t2.
	t4
		ifNil: [LDAPException new messageText: 'invalid tag -- make sure class is in identifyIncomingElements';
				 signal].
	t3 := t4 new setTag: t2.
	t3 decode: t1.
	^ t3! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
searchRequest: t1 base: t2 scope: t3 deref: t4 filter: t5 attrs: t6 wantAttrsOnly: t7 
	| t8 t9 t10 t11 |
	t8 := ReadWriteStream on: String new.
	t9 := BERSequence new.
	t9
		addElement: (BERInteger new value: t1).
	t10 := BERSequence new tagSetApplication.
	t10
		addElement: (BEROctetString new value: t2);
		
		addElement: (BEREnumerated new value: t3);
		
		addElement: (BEREnumerated new value: t4);
		
		addElement: (BERInteger new value: 0);
		
		addElement: (BERInteger new value: 0);
		
		addElement: (BERBoolean new value: t7);
		 addElement: t5 asBERElement tagSetContext withTag: t5 class tagValue.
	t11 := BERSequence new.
	t6
		do: [:t12 | t11
				addElement: (BEROctetString new value: t12)].
	t10 addElement: t11.
	t9 addElement: t10 withTag: 3.
	t9 writeOn: t8.
	^ t8 contents! !

!LDAPEncoder class methodsFor: 'as yet unclassified' stamp: ''!
unbindRequest: t1 
	| t2 t3 |
	t2 := ReadWriteStream on: String new.
	t3 := BERSequence new.
	t3
		addElement: (BERInteger new value: t1).
	t3 addElement: BERNull new tagSetApplication withTag: 2.
	t3 writeOn: t2.
	^ t2 contents! !

Object subclass: #LDAPFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
andOf: t1 
	| t2 |
	t2 := LDAPFilterAnd new.
	t1
		do: [:t3 | t2 add: t3].
	^ t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atAnywhereOf: t2 
	^ LDAPFilterSubstrings containing: t1 atAnywhereOf: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atEndOf: t2 
	^ LDAPFilterSubstrings containing: t1 atEndOf: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atStartOf: t2 
	^ LDAPFilterSubstrings containing: t1 atStartOf: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
not: t1 
	^ LDAPFilterNot filter: t1! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
orOf: t1 
	| t2 |
	t2 := LDAPFilterOr new.
	t1
		do: [:t3 | t2 add: t3].
	^ t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
with: t1 
	^ LDAPFilterPresent having: t1! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
with: t1 equalTo: t2 
	^ LDAPFilterEquality new setType: t1 value: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
with: t1 greaterOrEqualThan: t2 
	^ LDAPFilterGreaterOrEqual new setType: t1 value: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
with: t1 lessOrEqualThan: t2 
	^ LDAPFilterLessOrEqual new setType: t1 value: t2! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: ''!
with: t1 notEqualTo: t2 
	^ self
		not: (self with: t1 equalTo: t2)! !

Object subclass: #LDAPFilterSingle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterSingle class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	self subclassResponsibility! !

!LDAPFilterSingle methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	self subclassResponsibility! !

LDAPFilterSingle subclass: #LDAPFilterWithFilter
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

LDAPFilterWithFilter subclass: #LDAPFilterNot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterNot class methodsFor: 'as yet unclassified' stamp: ''!
filter: t1 
	^ self new setFilter: t1! !

!LDAPFilterNot class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 2! !

!LDAPFilterWithFilter methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 |
	t1 := BERSet new.
	t1 addElement: filter asBERElement tagSetContext withTag: filter class tagValue.
	^ t1! !

!LDAPFilterWithFilter methodsFor: 'as yet unclassified' stamp: ''!
setFilter: t1 
	filter := t1! !

LDAPFilterSingle subclass: #LDAPFilterWithSet
	instanceVariableNames: 'filters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

LDAPFilterWithSet subclass: #LDAPFilterAnd
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterAnd class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 0! !

LDAPFilterWithSet subclass: #LDAPFilterOr
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterOr class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 1! !

!LDAPFilterWithSet class methodsFor: 'as yet unclassified' stamp: ''!
new
	^ self basicNew initialize! !

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: ''!
add: t1 
	filters addLast: t1! !

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 |
	t1 := BERSet new.
	filters
		do: [:t2 | t1 addElement: t2 asBERElement tagSetContext withTag: t2 class tagValue].
	^ t1! !

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: ''!
initialize
	filters := OrderedCollection new! !

LDAPFilterSingle subclass: #LDAPFilterWithSubstring
	instanceVariableNames: 'type choiceTag substring'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

LDAPFilterWithSubstring subclass: #LDAPFilterSubstrings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atAnywhereOf: t2 
	^ self new
		setSubstring: t1
		type: t2
		choiceTag: 1! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atEndOf: t2 
	^ self new
		setSubstring: t1
		type: t2
		choiceTag: 2! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: ''!
containing: t1 atStartOf: t2 
	^ self new
		setSubstring: t1
		type: t2
		choiceTag: 0! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 4! !

!LDAPFilterWithSubstring methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 t2 |
	t1 := BERSequence new.
	t1
		addElement: (BEROctetString new value: type).
	t2 := BERSequence new.
	t2 addElement: (BEROctetString new value: substring) tagSetContext withTag: choiceTag.
	t1 addElement: t2.
	^ t1! !

!LDAPFilterWithSubstring methodsFor: 'as yet unclassified' stamp: ''!
setSubstring: t1 type: t2 choiceTag: t3 
	substring := t1.
	type := t2.
	choiceTag := t3! !

LDAPFilterSingle subclass: #LDAPFilterWithType
	instanceVariableNames: 'type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

LDAPFilterWithType subclass: #LDAPFilterPresent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterPresent class methodsFor: 'as yet unclassified' stamp: ''!
having: t1 
	^ self new setType: t1! !

!LDAPFilterPresent class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 7! !

!LDAPFilterWithType methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 |
	t1 := BEROctetString new value: type.
	^ t1! !

!LDAPFilterWithType methodsFor: 'as yet unclassified' stamp: ''!
setType: t1 
	type := t1! !

LDAPFilterSingle subclass: #LDAPFilterWithValueAssertion
	instanceVariableNames: 'type value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

LDAPFilterWithValueAssertion subclass: #LDAPFilterApprox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterApprox class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 8! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterEquality
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterEquality class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 3! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterGreaterOrEqual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterGreaterOrEqual class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 5! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterLessOrEqual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Filters'!

!LDAPFilterLessOrEqual class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 6! !

!LDAPFilterWithValueAssertion methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 |
	t1 := BERSequence new.
	t1
		addElement: (BEROctetString new value: type);
		
		addElement: (BEROctetString new value: value).
	^ t1! !

!LDAPFilterWithValueAssertion methodsFor: 'as yet unclassified' stamp: ''!
setType: t1 value: t2 
	type := t1.
	value := t2! !

Object subclass: #LDAPRequest
	instanceVariableNames: 'worker messageId waitingSemaphore isDone result isAbandoned'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

LDAPRequest subclass: #LDAPAbandonRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPAbandonRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 abandonId: t2 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder abandonRequest: messageId abandonId: t2)! !

LDAPRequest subclass: #LDAPAddRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPAddRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 dn: t2 attrs: t3 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder
				addRequest: messageId
				dn: t2
				attrs: t3)! !

LDAPRequest subclass: #LDAPBindRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPBindRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 username: t2 credentials: t3 method: t4 
	t4
		ifNotNil: [self notYetImplemented].
	self initializeWith: t1.
	t1
		send: (LDAPEncoder
				bindRequest: messageId
				username: t2
				credentials: t3
				method: t4)! !

LDAPRequest subclass: #LDAPCompareRequest
	instanceVariableNames: 'searchResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPCompareRequest methodsFor: 'as yet unclassified' stamp: ''!
checkForExceptions
	self isDone
		ifFalse: [^ self error: 'LDAP timeout'].
	searchResult checkForExceptions! !

!LDAPCompareRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 dn: t2 comparing: t3 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder
				compareRequest: messageId
				dn: t2
				comparing: t3)! !

!LDAPCompareRequest methodsFor: 'as yet unclassified' stamp: ''!
processResponse: t1 
	isDone := true.
	searchResult := t1.
	t1 resultCode = 5
		ifTrue: [result := false].
	t1 resultCode = 6
		ifTrue: [result := true].
	waitingSemaphore signal! !

LDAPRequest subclass: #LDAPDelRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPDelRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 dn: t2 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder delRequest: messageId dn: t2)! !

LDAPRequest subclass: #LDAPModifyDNRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyDNRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 modifyDN: t2 toRDN: t3 underDN: t4 wantToClearAttrs: t5 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder
				modifyDNRequest: messageId
				dn: t2
				toRDN: t3
				underDN: t4
				wantToClearAttrs: t5)! !

LDAPRequest subclass: #LDAPModifyOp
	instanceVariableNames: 'attr values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

LDAPModifyOp subclass: #LDAPModifyAddOp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyAddOp class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 0! !

LDAPModifyOp subclass: #LDAPModifyDelOp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyDelOp class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 1! !

!LDAPModifyOp class methodsFor: 'as yet unclassified' stamp: ''!
affecting: t1 values: t2 
	| t3 |
	t3 := self basicNew initializeWithAttr: t1 values: t2.
	^ t3! !

!LDAPModifyOp class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ self subclassResponsibility! !

!LDAPModifyOp methodsFor: 'as yet unclassified' stamp: ''!
asBERElement
	| t1 |
	t1 := BERSequence new.
	t1
		addElement: (BEREnumerated new value: self class tagValue).
	t1
		addElement: (LDAPEncoder
				attrAsBERElement: (Association key: attr value: values)).
	^ t1! !

!LDAPModifyOp methodsFor: 'as yet unclassified' stamp: ''!
initializeWithAttr: t1 values: t2 
	attr := t1.
	values := t2! !

!LDAPModifyOp methodsFor: 'as yet unclassified' stamp: ''!
printOn: t1 
	t1 nextPutAll: self class name , '(' , attr , ': ' , values asArray , ')'! !

LDAPModifyOp subclass: #LDAPModifyReplaceOp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyReplaceOp class methodsFor: 'as yet unclassified' stamp: ''!
tagValue
	^ 2! !

LDAPRequest subclass: #LDAPModifyRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPModifyRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 dn: t2 ops: t3 
	self initializeWith: t1.
	t1
		send: (LDAPEncoder
				modifyRequest: messageId
				dn: t2
				ops: t3)! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
abandon
	isAbandoned
		ifFalse: [LDAPAbandonRequest new initializeWith: worker abandonId: messageId.
			worker delTargetFor: messageId.
			isAbandoned := true]! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
checkForExceptions
	self isDone
		ifFalse: [^ self error: 'LDAP timeout'].
	result checkForExceptions! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
defaultTimeout
	^ 30! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
hasResult
	^ unknown3! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 
	worker := t1.
	messageId := worker nextMessageId.
	waitingSemaphore := Semaphore new.
	isAbandoned := false.
	isDone := false.
	worker addTarget: self for: messageId! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
isDone
	^ unknown3! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
processResponse: t1 
	isDone := true.
	result := t1.
	waitingSemaphore signal! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
result
	self wait.
	^ result! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
wait
	^ self waitTimeoutSeconds: self defaultTimeout! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
waitTimeoutMSecs: t1 
	result
		ifNil: [waitingSemaphore waitTimeoutMSecs: t1].
	self checkForExceptions! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
waitTimeoutSeconds: t1 
	result
		ifNil: [waitingSemaphore waitTimeoutSeconds: t1].
	self checkForExceptions! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: ''!
wantsMoreResponses
	^ self isDone not! !

LDAPRequest subclass: #LDAPSearchRequest
	instanceVariableNames: 'partialResults iterator gotNewResultSemaphore searchResult resultCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: ''!
checkForExceptions
	self isDone
		ifFalse: [^ self error: 'LDAP timeout'].
	searchResult resultCode = 32
		ifFalse: [searchResult checkForExceptions]! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 base: t2 scope: t3 deref: t4 filter: t5 attrs: t6 wantAttrsOnly: t7 
	| t8 |
	partialResults := OrderedCollection new.
	iterator := 0.
	resultCount := 0.
	gotNewResultSemaphore := Semaphore new.
	self initializeWith: t1.
	t8 := LDAPEncoder
				searchRequest: messageId
				base: t2
				scope: t3
				deref: t4
				filter: t5
				attrs: t6
				wantAttrsOnly: t7.
	t1 send: t8! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: ''!
next
	^ self nextTimeout: self defaultTimeout! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: ''!
nextTimeout: t1 
	isDone
		ifFalse: [gotNewResultSemaphore waitTimeoutSeconds: t1.
			(resultCount = iterator
					and: [isDone not])
				ifTrue: [^ self error: 'LDAP timeout']].
	resultCount > iterator
		ifTrue: [^ partialResults at: (iterator := iterator + 1)].
	^ nil! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: ''!
processResponse: t1 
	t1 class isEndMarker
		ifTrue: [searchResult := t1.
			isDone := true.
			result := partialResults.
			waitingSemaphore signal]
		ifFalse: [partialResults addLast: t1.
			resultCount := resultCount + 1].
	gotNewResultSemaphore signal! !

LDAPRequest subclass: #LDAPUnbindRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPUnbindRequest methodsFor: 'as yet unclassified' stamp: ''!
initializeWith: t1 
	| t2 |
	worker := t1.
	messageId := worker nextMessageId.
	t2 := LDAPEncoder unbindRequest: messageId.
	worker send: t2! !

Object subclass: #LDAPWorker
	instanceVariableNames: 'sockStream hostname port nextMessageId lock targets running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!LDAPWorker class methodsFor: 'as yet unclassified' stamp: ''!
newTo: t1 port: t2 
	^ self basicNew initialize: t1 port: t2! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
addTarget: t1 for: t2 
	| t3 |
	lock
		critical: [targets
				at: t2
				ifPresent: [Error new; messageText: 'messageId already present -- shouldnt happen'; signal].
			t3 := Dictionary new.
			t3 at: #request put: t1.
			targets at: t2 put: t3]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
connect
	[sockStream := SocketStream openConnectionToHostNamed: hostname port: port]
		on: Exception
		do: [:t1 | 
			sockStream := nil.
			self error: t1 messageText].
	sockStream binary; autoFlush: false; noTimeout! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
delTargetFor: t1 
	lock
		critical: [targets
				at: t1
				ifAbsent: [^ self].
			targets removeKey: t1]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
disconnect
	self stop! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
dispatchMessage: t1 
	| t2 t3 t4 |
	t3 := t1 id.
	t3 = 0
		ifTrue: [^ self].
	t2 := targets
				at: t3
				ifAbsent: [^ self].
	t2
		ifNil: [self delTargetFor: t3.
			^ self].
	t4 := t2 at: #request.
	t4 processResponse: t1.
	t4 wantsMoreResponses
		ifFalse: [self delTargetFor: t3]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
initialize: t1 port: t2 
	hostname := t1.
	port := t2.
	nextMessageId := 0.
	lock := Semaphore forMutualExclusion.
	targets := Dictionary new.
	running := false! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
nextMessageId
	nextMessageId := nextMessageId + 1.
	^ nextMessageId! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
running
	^ unknown6! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
running: t1 
	running := t1! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
send: t1 
	lock
		critical: [sockStream nextPutAll: t1;
				 flush]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
start
	| t1 t2 |
	running := true.
	t1 := [[running]
				whileTrue: [t2 := [LDAPMessage newFrom: sockStream]
								on: Error
								do: [:t3 | 
									Transcript cr; show: t3 description.
									running := false].
					running
						ifTrue: [self dispatchMessage: t2]].
			nil] fork.
	^ t1! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: ''!
stop
	running := false! !

Object subclass: #RecursionLock
	instanceVariableNames: 'lockedByProcess semaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Core'!

!RecursionLock class methodsFor: 'instance creation' stamp: ''!
new
	^ self basicNew initialize! !

!RecursionLock methodsFor: 'mutual-exclusion' stamp: ''!
critical: t1 
	| t2 |
	lockedByProcess = Processor activeProcess
		ifTrue: [t2 := t1 value]
		ifFalse: [semaphore
				critical: [lockedByProcess := Processor activeProcess.
					t2 := [t1 value]
								ensure: [lockedByProcess := nil]]].
	^ t2! !

!RecursionLock methodsFor: 'initialize-release' stamp: ''!
initialize
	semaphore := Semaphore forMutualExclusion! !

Error subclass: #LDAPException
	instanceVariableNames: 'resultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!

LDAPException subclass: #LDAPAttrProblemException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!

!LDAPException class methodsFor: 'as yet unclassified' stamp: ''!
newWithCode: t1 
	| t2 t3 |
	t3 := self.
	(t1 >= 16
			and: [t1 <= 21])
		ifTrue: [t3 := LDAPAttrProblemException].
	(t1 >= 32
			and: [t1 <= 36]
			and: [t1 ~= 35])
		ifTrue: [t3 := LDAPNameProblemException].
	(t1 >= 48
			and: [t1 <= 50])
		ifTrue: [t3 := LDAPSecurityProblemException].
	(t1 >= 51
			and: [t1 <= 54])
		ifTrue: [t3 := LDAPServiceProblemException].
	((t1 >= 64
				and: [t1 <= 69])
			or: [t1 = 71])
		ifTrue: [t3 := LDAPUpdateProblemException].
	t2 := t3 new initializeWithCode: t1.
	^ t2! !

!LDAPException methodsFor: 'as yet unclassified' stamp: ''!
asSymbol
	^ resultCode caseOf: {
		[0] -> [#LDAPSuccess].
		[1] -> [#LDAPOperationsError].
		[2] -> [#LDAPProtocolError].
		[3] -> [#LDAPTimeLimitExceeded].
		[4] -> [#LDAPSizeLimitExceeded].
		[5] -> [#LDAPCompareFalse].
		[6] -> [#LDAPCompareTrue].
		[7] -> [#LDAPAuthMethodNotSupported].
		[8] -> [#LDAPStrongAuthRequired].
		[10] -> [#LDAPReferral].
		[11] -> [#LDAPAdminLimitExceeded].
		[12] -> [#LDAPUnavailableCriticalExtension].
		[13] -> [#LDAPConfidentialityRequired].
		[14] -> [#LDAPSaslBindInProgress].
		[16] -> [#LDAPNoSuchAttribute].
		[17] -> [#LDAPUndefinedAttributeType].
		[18] -> [#LDAPInappropriateMatching].
		[19] -> [#LDAPConstraintViolation].
		[20] -> [#LDAPAttributeOrValueExists].
		[21] -> [#LDAPInvalidAttributeSyntax].
		[32] -> [#LDAPNoSuchObject].
		[33] -> [#LDAPAliasProblem].
		[34] -> [#LDAPInvalidDnSyntax].
		[35] -> [#LDAPIsLeaf].
		[36] -> [#LDAPAliasDereferencingProblem].
		[48] -> [#LDAPInappropriateAuthentication].
		[49] -> [#LDAPInvalidCredentials].
		[50] -> [#LDAPInsufficientAccessRights].
		[51] -> [#LDAPBusy].
		[52] -> [#LDAPUnavailable].
		[53] -> [#LDAPUnwillingToPerform].
		[54] -> [#LDAPLoopDetect].
		[64] -> [#LDAPNamingViolation].
		[65] -> [#LDAPObjectClassViolation].
		[66] -> [#LDAPNotAllowedOnNonleaf].
		[67] -> [#LDAPNotAllowedOnRdn].
		[68] -> [#LDAPEntryAlreadyExists].
		[69] -> [#LDAPObjectClassModsProhibited].
		[71] -> [#LDAPAffectsMultipleDsas].
		[80] -> [#LDAPOther].
		[81] -> [#LDAPServerDown].
		[82] -> [#LDAPLocalError].
		[83] -> [#LDAPEncodingError].
		[84] -> [#LDAPDecodingError].
		[85] -> [#LDAPLdapTimeout].
		[86] -> [#LDAPAuthUnknown].
		[87] -> [#LDAPFilterError].
		[88] -> [#LDAPUserCancelled].
		[90] -> [#LDAPNoMemory].
		[91] -> [#LDAPConnectError].
		[92] -> [#LDAPLdapNotSupported].
		[93] -> [#LDAPControlNotFound].
		[94] -> [#LDAPNoResultsReturned].
		[95] -> [#LDAPMoreResultsToReturn].
		[96] -> [#LDAPClientLoop].
		[97] -> [#LDAPReferralLimitExceeded].
		[100] -> [#LDAPInvalidResponse].
		[101] -> [#LDAPAmbiguousResponse].
		[112] -> [#LDAPTlsNotSupported]}
		 otherwise: [#LDAPUnknownReturnCode]! !

!LDAPException methodsFor: 'as yet unclassified' stamp: ''!
initializeWithCode: t1 
	resultCode := t1.
	self messageText: self asSymbol! !

LDAPException subclass: #LDAPNameProblemException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!

LDAPException subclass: #LDAPSecurityProblemException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!

LDAPException subclass: #LDAPServiceProblemException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!

LDAPException subclass: #LDAPUpdateProblemException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Exceptions'!
